function [plantmass,soilchem_pred,residchem,residdepth,residgroup] = verhib(varargin)
%
% [plantmass,soilchem_pred,residchem,residdepth,residgroup] = ...
%     verhib(inputxls,regdepth,reggroup)
%
%   or
%
% [plantmass,soilchem_pred,residchem,residdepth,residgroup] = ...
%      verhib(soilchem,leafchem,rootchem,leafdepth,rootdepth, ...
%             plantgroups,leafrootratio,yearperlayer,regdepth,reggroup);
%
% e.g.
%
% verhib('verhib_g15',0.1,0.1)
%

if isstr( varargin{1} )

    xlsInOut = true;
    inputxls = varargin{1};
    regdepth = varargin{2};
    reggroup = varargin{3};

    [soilchem,leafchem,rootchem,leafdepth,rootdepth,plantgroups,useleaf,useroot,leafrootratio,yearperlayer,nrcomp,nrlayers,nrplants,nrgroups,depthlbl_num,species,chemlbl] = read_verhib_xls(inputxls);

else

    soilchem = varargin{1};
    leafchem = varargin{2};
    rootchem = varargin{3};
    leafdepth = varargin{4};
    rootdepth = varargin{5};
    plantgroups = varargin{6};
    leafrootratio = varargin{7};
    yearperlayer = varargin{8};
    regdepth = varargin{9};
    reggroup = varargin{10};

end


%% build matrices for regression function

[E,f,G,h,C,d,minmass,maxmass] = mk_verhib_matrices(soilchem,leafchem,rootchem,leafdepth,rootdepth,plantgroups,useleaf,useroot,leafrootratio,yearperlayer,regdepth,reggroup);


%% the actual regression

% set tolerance and maxnr of iterations to reasonable levels for this
% problem
options = optimset('TolFun',1e-1,'MaxIter',1000);
[par,resnorm,residual,exitflag] = lsqlin( E, f, G, h, C, d, minmass, maxmass, maxmass/nrplants, options );
%exitflag

% [X,RESNORM,RESIDUAL,EXITFLAG] = LSQLIN(C,d,A,b,Aeq,beq,LB,UB) 
%
% Solves the least-squares, with equality constraints problem:
% 
%            min  0.5*(NORM(C*x-d)).^2     
%             x                          
%
%            subject to: A*x <= b and Aeq*x = beq
%     
%     so that the solution is in the range LB <= X <= UB. Use empty matrices for 
%     LB and UB if no bounds exist. Set LB(i) = -Inf if X(i) is unbounded 
%     below; set UB(i) = Inf if X(i) is unbounded above.
    
% par = lsie(E,f,G,h,C,d);
% this minimizes ||Ex - f|| subject to Gx >= h and Cx = d.

%% reformat output
[plantmass,soilchem_pred,residchem,residdepth,residgroup] = mk_verhib_output(nrcomp,nrlayers,nrplants,nrgroups,E,f,par,leafrootratio);

%% write output


if xlsInOut
    write_verhib_xls(inputxls,regdepth,reggroup,depthlbl_num,species,chemlbl,plantmass,residchem)
else
    disp('Output not written to xls-file')

end



% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% function [soilchem,leafchem,rootchem,leafdepth,rootdepth,plantgroups,useleaf,useroot,leafrootratio,yearperlayer,nrcomp,nrlayers,nrplants,nrgroups,depthlbl_num,species,chemlbl] = read_verhib_xls(inputxls)
% 
%     %% load data from 'inputxls'
% 
%     [dat,lbl] = xlsread( inputxls, 'includechem' );
%     % endcol = size(dat,2);
%     inchem = logical( [0 dat] );
%     chemlbl = lbl(1,inchem);    
% 
%     [dat,lbl] = xlsread( inputxls, 'plantgroups' );
%     useleaf = logical( dat(:,6) );
%     useroot = logical( dat(:,7) );
%     inplant = useleaf | useroot;
%     useleaf = logical( dat(inplant,6) )';
%     useroot = logical( dat(inplant,7) )';
%     leafrootratio = dat( inplant, end )';
% 
%     plantgroups = dat( inplant, 2:end-1 )';
%     inplantlbl = [false; inplant];
% 
%     family = lbl( inplantlbl, 1 );
%     genus = lbl( inplantlbl, 2);
%     species =  lbl( inplantlbl, 3 );
% 
%     [dat,lbl] = xlsread( inputxls, 'depthdur' );
%     yearperlayer = dat(:,2);
% 
%     [dat,lbl] = xlsread( inputxls, 'soilchem' );
%     soilchem = dat( :, inchem )';
%     depthlbl_num = dat(:,1);
% 
%     [dat,lbl] = xlsread( inputxls, 'leafchem' );
%     leafchem = dat( inplant, inchem )';
% 
%     [dat,lbl] = xlsread( inputxls, 'rootchem' );
%     rootchem = dat( inplant, inchem )';
% 
%     [dat,lbl] = xlsread( inputxls, 'leafdepth' );
%     leafdepth = dat( inplant, 2:end )';
%     depthlbl_str = lbl( 1, 5:end );
% 
%     [dat,lbl] = xlsread( inputxls, 'rootdepth' );
%     rootdepth = dat( inplant, 2:end )';
% 
%     % rootchem = [];
%     % rootdepth =[];
% 
%     % determine the dimension of the problem
%     [nrcomp,nrlayers] = size(soilchem);
%     nrplants = size(leafchem,2);
% 
%     % remove plantgroups without plants
%     plantsingroup = sum(plantgroups,2)>0;
%     plantgroups = plantgroups(plantsingroup,:);
%     nrgroups = size( plantgroups, 1 );
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% function [E,f,G,h,C,d,minmass,maxmass] = mk_verhib_matrices(soilchem,leafchem,rootchem,leafdepth,rootdepth,plantgroups,leafrootratio,yearperlayer,regdepth,reggroup,depthlimit)
% % where
% %   soilchem      is the matrix with soil chemical data (nrcomp,nrlayers)
% %   leafchem      is the matrix with leaf chemical data (nrcomp,nrplants)
% %   rootchem      is the matrix with root chemical data (nrcomp,nrplants)
% %   leafdepth     is the matrix with leaf depth relation (nrlayers,nrplants)
% %   rootdepth     is the matrix with leaf depth relation (nrlayers,nrplants)
% %   plantgroups   is the matrix with plant groups (nrgroups,nrplants) can be left empty
% %   leafrootratio is the (1,nrplants) vector with the ratio leafmass:rootmass per plant
% %   yearperlayer  is the (nrlayer,1) vector with the nr of years per depth layer
% %   regdepth      is the regularization to weight the importance of stiffness over time 
% %                 parameter (usually a value between 0
% %                 and 0.1, smaller means less regularization)
% %   reggroups     is the regularization parameter for plant groups
% %   depthlimit    specifies the number of layers to consider for spatial partitioning 
% %                 - to make the problem easier to sovle
% %
% % output-matrices and vectors are for the least squares problem:
% % ||Ex - f|| subject to Gx >= h and Cx = d
% %
% % the plantmass is accounted for in Gx >= h
% % and the leaf-root ratio in Cx = d
% % 
% 
% % for a case with 2 plants, and three layers,
% % and including the root info the resulting X
% % matrix should look like:
% %
% %            leafs                     roots
% %      <- nrplants*nrlayers ->   <- nrplants*nrlayers ->
% %
% % X = [ a1  a2  0   0   0   0    c1  c2  0   0   0   0       ^
% %       b1  b2  0   0   0   0    d1  d2  0   0   0   0       |
% %       0   0   a1  a2  0   0    0   0   c1  c2  0   0       | nrcomp * nrlayers
% %       0   0   b1  b2  0   0    0   0   d1  d2  0   0       |
% %       0   0   0   0   a1  a2   0   0   0   0   c1  c2      |
% %       0   0   0   0   b1  b2   0   0   0   0   d1  d2      V
% %       .9  0   0   0   0   0    .7  0   0   0   0   0       ^    
% %       0   .8  0   0   0   0    0   .6  0   0   0   0       |
% %       .1  0   .9  0   0   0    .2  0   .7  0   0   0       |
% %       0   .2  0   .8  0   0    0   .3  0   .6  0   0       | nrplants * nrlayers
% %       0   0   .1  0   .9  0    .1  0   .2  0   .7  0       | 
% %       0   0   0   .2  0   .8   0   .1   0  .3  0   .6      |
% %       .1  .1  0   0   0   0    .1  .1  0   0   0   0       ^
% %       0   0   .1  .1  0   0    0   0   .1  .1  0   0       | nrgroup * nrlayers
% %       0   0   0   0   .1  .1   0   0   0   0   .1  .1      V
% %
% %
% 
% if ~exist('depthlimit','var')
%    depthlimit = size(leafdepth,1);
% end
% 
% if ~exist('reggroups','var') || isempty(reggroups)
%    reggroups = 0.1;
% end
% 
% if ~exist('regdepth','var') || isempty(regdepth)
%    regdepth = 0.1;
% end
% 
% maxleafperyear = 1000; % (g per m2)
% maxrootperyear = 1000; % (g per m2)
% 
% leafrootratio = leafrootratio(:)';
% 
% % determine the dimension of the problem
% [nrcomp,nrlayers] = size(soilchem);
% nrplants = size(leafchem,2);
% 
% % remove plantgroups without plants
% plantsingroup = sum(plantgroups,2)>0;
% plantgroups = plantgroups(plantsingroup,:);
% nrgroups = size( plantgroups, 1 );
%  
% % leafpart = leafrootratio./(leafrootratio+1);
% % rootpart = 1-leafpart;
% 
% if isempty(rootdepth) || isempty(rootchem)
%     
%     % build the y-vector
%     ytime = zeros(nrlayers*nrplants-nrplants,1);
%     
%     if isempty(plantgroups)
%         ygroups = [];
%     else
%         ygroups = reggroups * repmat(sum(plantgroups,2),nrlayers,1);
%     end
%     f = [ soilchem(:); ytime; ygroups];
% 
%     % build the x-matrix
% 
%     leafblocks = repmat( leafchem, nrlayers,nrlayers ) .* ...
%                  spacetime_knitting( leafdepth./100, nrcomp, depthlimit );
% 
%     % the blocks to control time-stiffness distribution
%     timebands = - eye(nrplants*nrlayers) + diag( ones( (nrplants*nrlayers - nrplants) ,1),-nrplants );
%     timebands = timebands( nrplants+1:end , : );
% 
%     if isempty(plantgroups)
%         pgroupblocks = [];
%     else
%         pgroupblocks = blockdiag_lim(plantgroups,nrlayers);
%     end
% 
%     E = [ leafblocks; regdepth*timebands; reggroups*pgroupblocks ];
% 
%     totperlayer = blockdiag_lim( ones(1,nrplants), nrlayers );
%     totleafperlayer = yearperlayer * maxleafperyear;
% %     G = [ eye(nrplants*nrlayers); -totperlayer ];
% %     h = [ zeros(nrplants*nrlayers,1); -totleafperlayer ];
%     G = totperlayer;
%     h = totleafperlayer;
% 
%     C = [];
%     d = [];
%     
% else  % leaves and roots
%     
%     % build the y-vector
%     ytime = zeros(2*nrlayers*nrplants - 2*nrplants,1);
%     
%     if isempty(plantgroups)
%         ygroups = [];
%     else
%         ygroups = reggroups * repmat(sum(plantgroups,2),2*nrlayers,1);
%     end
%     
% %     yleafroot = zeros(nrplants*nrlayers,1);
%     
%     f = [ soilchem(:); ytime; ygroups];
% 
%     
%     % the blocks for the biomarker mass in the plant parts
%     
%     leafblocks = repmat(leafchem,nrlayers,nrlayers) .* ...
%                  spacetime_knitting(leafdepth./100, nrcomp, depthlimit );
%              
%     rootblocks = repmat(rootchem,nrlayers,nrlayers) .* ...
%                  spacetime_knitting(rootdepth./100, nrcomp, depthlimit );
% 
%     totalmass = [leafblocks rootblocks];
%     
%     % the blocks to control time-stiffness distribution
%     timebands = - eye(nrplants*nrlayers) + diag( ones( (nrplants*nrlayers - nrplants) ,1),-nrplants );
%     timebands = timebands( nrplants+1:end , : );
%     zeroblock = zeros(nrplants*nrlayers - nrplants,nrplants*nrlayers);
%     totaltimebands = [timebands zeroblock; zeroblock timebands];
% 
%     % the blocks for the plant groups
%     if isempty(plantgroups)
%         pgroupblocks = [];
%     else
%         pgroupblocks = blockdiag_lim(plantgroups,nrlayers);
%     end
%     zeroblock = zeros(nrgroups*nrlayers,nrplants*nrlayers);
%     totalpgroupblocks = [pgroupblocks zeroblock; zeroblock pgroupblocks];
%     
% %     % the blocks for the leafroot-ratio
% %     leafrootblock =  [ eye(nrplants*nrlayers),  - diag( repmat(leafrootratio',nrlayers,1) ) ];
%             
%     E = [ totalmass              
%           regdepth *  totaltimebands 
%           reggroups * totalpgroupblocks ];
% 
%     totperlayer = blockdiag_lim( ones(1,nrplants), nrlayers );
%     totplantperlayer = yearperlayer * ( maxleafperyear + maxrootperyear );
% 
% %  de pijn zit hem er misschien in dat wortels en blad achter
% %  elkaar staan (en gezamenlijk > dan 0 moeten zijn)
% %  terwijl ze dat afzonderlijk ook zijn
% %
% %     G = [ eye(nrplants*nrlayers) eye(nrplants*nrlayers); 
% %           -totperlayer           -totperlayer  ];
% %     h = [ zeros(nrplants*nrlayers,1); -totplantperlayer ];
% 
%     G = [ totperlayer, totperlayer ];
%     h = totplantperlayer;
% 
%     C = [ eye(nrplants*nrlayers) ,  diag( repmat( -leafrootratio(:), nrlayers, 1) ) ];
%     d = zeros(nrplants*nrlayers,1);
% 
% end
% 
% nrpar = size(E,2);
% minmass = zeros(nrpar,1);
% maxmass = ones( nrpar/nrlayers,1 ) * h' ;  % note: h is the maximum biomass production per layer
% maxmass = maxmass(:);
% 
% 
% %%
% function [partitionmatrix] = spacetime_knitting(partition_in,nrcomp,nrlag)
% 
% [nrlayer, nrplant] = size(partition_in);
% 
% mindim = min(nrlayer,nrlag);
% 
% partitionmatrix = zeros(nrcomp*nrlayer,nrplant*mindim);
% 
% for diagonal = 1:mindim
% 
%     i=0;
%     j=0;
% 
%     for n = 1:nrlayer + 1 - diagonal
%         partition_submatrix = [ repmat( partition_in(diagonal,:),nrcomp,1 ) ];
% 
%         rl = (diagonal-1)*nrcomp + i + 1 : (diagonal-1)*nrcomp + i + nrcomp;
%         cl = j+1:j+nrplant;
%         partitionmatrix(rl,cl) = partition_submatrix ;
% 
%         i = i+nrcomp;
%         j = j+nrplant;
%     end
% 
% end
% 
% %%
% % [output] = blockdiag_lim(input,nrblk)
% %
% % create a block diagonal matrix
% % using nrblk blocks
% %
% % Emiel van Loon
% 
% function [output] = blockdiag_lim(input,nrblk)
% 
% [nrow,ncol]=size(input);
% output = zeros(nrow*nrblk,ncol*nrblk);
% 
% i=0;
% j=0;
% for k = 1:nrblk
%     output(i+1:i+nrow,j+1:j+ncol)=input;
%     i = i+nrow;
%     j = j+ncol;
% end
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% function [plantmass,soilchem_pred,residchem,residdepth,residgroup] = mk_verhib_output(nrcomp,nrlayers,nrplants,nrgroups,X,y,par,leafrootratio)
% 
% % totale lengte van y is:
% %   nrcomp*nrlayers + nrplants*(nrlayers-1) + nrgroups*nrlayers
% %
% 
% residual = y - X*par;
% 
% if length(par)==nrplants*nrlayers   % only leaves were considered - no roots
% 
%     totmass = par + par./repmat(leafrootratio(:),nrlayers,1);
%     plantmass = reshape( totmass, nrplants, nrlayers );
% 
%     soilchem_pred = reshape(y(1:nrcomp*nrlayers),nrcomp,nrlayers);
% 
%     residchem = reshape(residual(1:nrcomp*nrlayers),nrcomp,nrlayers);
% 
%     brk = nrcomp*nrlayers;
% 
%     residdepth = reshape( residual( brk + 1 : brk + nrplants*(nrlayers-1) ), nrplants, nrlayers-1);
% 
%     brk = nrcomp*nrlayers + nrplants*(nrlayers-1);
% 
%     residgroup = reshape( residual( brk + 1 : brk + nrgroups*nrlayers ), nrgroups, nrlayers);
% 
% else  % both leaves and roots were considered.
% 
% 
%     % reshape the parameter output and calculate the relative contribution
%     % of each plant to the total
%     plantmass = reshape( par( 1 : nrplants*nrlayers ), nrplants, nrlayers ) + ...
%         reshape( par( nrplants*nrlayers+1 : 2*nrplants*nrlayers ), nrplants, nrlayers ) ;
%     % plantmass_relative = plantmass ./ repmat( sum(plantmass) ,nrplants,1 );
% 
%     % should be the same
%     % rootpar = reshape( par(nrplants*nrlayers+1:end), nrplants, nrlayers );
%     % rootpar_relative = rootpar ./ repmat( sum(rootpar) ,nrplants,1 );
% 
%     % reshape the predicted biomarker concentration to the right format:
%     % after 'reshape' each row is one component, each column is one layer,
%     % this is transposed.
%     % the same for the residuals and regularized values
%     soilchem_pred = reshape(y(1:nrcomp*nrlayers),nrcomp,nrlayers);
% 
%     residchem = reshape(residual(1:nrcomp*nrlayers),nrcomp,nrlayers);
% 
%     brk = nrcomp*nrlayers;
% 
%     residdepth = reshape( residual( brk + 1 : brk + nrplants*nrlayers ), nrplants, nrlayers);
% 
%     brk = nrcomp*nrlayers + nrplants*nrlayers;
% 
%     residgroup = reshape( residual( brk + 1 : brk + nrgroups*nrlayers ), nrgroups, nrlayers);
% 
% end
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% function [x,phi] = lsie(E,f,G,h,C,d)
% %LSIE   Least Squares with linear Inequalities and Equality constraints.
% %   [x,phi] = lsie(E,f,G,h,C,d) solves the following least squares problem:
% %
% %      minimize ||Ex - f|| subject to Gx >= h and Cx = d.
% %
% % if C and d are not supplied then the following is solved
% %      minimize ||Ex - f|| subject to Gx >= h
% %
% % if G and h are not supplied, then the following is solved
% %      minimize ||Ex - f|| subject to x >= 0
% 
% % Algorithm:
% %  LSIE reduces the number of independent variables with the constraint
% %  equations (Cx=d) and then uses LSI to solve the remaining system. See
% %  Lawson and Hanson (1974, pg 168-9).
% 
% % Reference:
% %  Lawson and Hanson, "Solving Least Squares Problems", Prentice-Hall, 1974.
% %  Chapter 23 & 20.
% 
% if nargin == 6 && ~isempty(C) && ~isempty(d) && ~isempty(G) && ~isempty(h)
% 
%     [q,r] = qr(C');
%     K = q;
% 
%     CEG = [C; E; G];
%     % CEGK = CEG * K
% 
%     CK = C * K;
%     EK = E * K;
%     GK = G * K;
% 
%     % CEGK - [CK; EK; GK]
% 
%     [mc,nc] = size(C);
%     [me,ne] = size(E);
%     [mg,ng] = size(G);
% 
%     C1 = CK(:,1:mc);
% 
%     E1 = EK(:,1:mc);
%     E2 = EK(:,mc+1:nc);
% 
%     G1 = GK(:,1:mc);
%     G2 = GK(:,mc+1:nc);
% 
% 
%     y1 = C1\d;
% 
%     [y2,phi] = lsi(E2,(f-E1*y1),G2,(h-G1*y1));
% 
%     x = K * [y1; y2];
% 
% elseif ( nargin==4 && ~isempty(G) && ~isempty(h) ) ||...
%        ( ( isempty(C) || isempty(d) ) && ~isempty(G) && ~isempty(h) )
% 
%     [x,phi] = lsi(E,f,G,h);
% 
% elseif nargin==2 || ( isempty(C) || isempty(d) )
% 
%     [x,phi] = nnls(E,f);
% 
% end
% 
% %%
% function [x,phi] = lsi(E,f,G,h)
% %LSI   Least Squares with linear Inequalities.
% %   [x,phi] = lsi(E,f,G,h) solves the following least squares problem:
% %
% %      minimize ||Ex - f|| subject to Gx >= h
% %
% %
% %   See also NNLS, LDP.
% %
% 
% % Algorithm:
% %  LSI converts the constrained least squares problem into an LDP problem.
% %  See Lawson and Hanson (1974, pg 167-8).
% 
% % Reference:
% %  Lawson and Hanson, "Solving Least Squares Problems", Prentice-Hall, 1974.
% %  Chapter 23.
% 
% if nargin ~= 4, error('Incorrect number of input arguments.'); end
% 
% [m,n] = size(E);
% 
% % E K = Q R
% % note can also use SVD here
% [Q,R,K] = qr(E);
% %[Q,R,K] = svd(E);
% 
% Q1 = Q(:,1:n);
% f1 = Q1' * f;
% 
% GKinvR = G * K * inv(R(1:n,1:n));
% 
% [z,phi] = ldp(GKinvR, h - GKinvR * f1);
% 
% 
% x = K * inv(R(1:n,1:n)) * (z + f1);
% 
% 
% %%
% 
% function [x,phi] = ldp(G,h)
% %LDP   Least Distance Programming.
% %   [x,phi] = ldp(G,h) solves the following least squares problem:
% %
% %      minimize ||x|| subject to Gx >= h
% %
% %
% %   See also NNLS, LSI.
% %
% 
% % Algorithm:
% %  (1) Define the (n+1) x m matrix E and the (n+1)-vector f as
% %      E := [G' h']' and f := [0,...,0,1]'. Use Algorithm
% %      NNLS to compute an m-vector u_hat solving Problem NNLS:
% %      Minimize ||Eu-f|| subject to u >= 0.
% %  (2) Compute the (n+1)-vector r := Eu_hat - f.
% %  (3) If ||r|| = 0, set phi := FALSE and goto Step 6.
% %  (4) Set phi := TRUE.
% %  (5) For j := 1,...,n, compute x_hat_j := -r_j / r_{n+1}
% %  (6) The computation is complete.
% %
% %
% 
% % Reference:
% %  Lawson and Hanson, "Solveing Least Squares Problems", Prentice-Hall, 1974.
% %  Chapter 23.
% 
% if nargin ~= 2, error('Incorrect number of input arguments.'); end
% 
% % initialize variables
% 
% [m,n] = size(G);
% E = [G'; h'];
% f = cat(2,zeros(1,n),[1])';
% 
% 
% % ||Eu - f|| subject to u >= 0
% u_hat = nnls(E,f);
% 
% 
% r = E * u_hat - f;
% 
% 
% if (norm(r) == 0),
%     phi = 0;
%     x_hat = NaN;
% else
%     phi = 1;
%     x_hat = -r / r(n+1);
% end
% 
% x = x_hat(1:n);
% 
% 
% %%
% 
% function [x,w] = nnls(X,y,tol)
% %NNLS	Non-negative least-squares.
% %
% % 	Adapted from NNLS of Mathworks, Inc.
% %
% %	p = nnls(X,y) returns the vector p that solves p = pinv(X'*X)*X'*y
% %	in a least squares sense, subject to p >= 0.
% %	Differently stated it solves the problem min ||y - X*p|| if
% %	XtX = X'*X and Xty = X'*y.
% %
% %	A default tolerance of TOL = MAX(SIZE(X'*X)) * NORM(X'*X,1) * EPS
% %	is used for deciding when elements of x are less than zero.
% %	This can be overridden with p = nnls(X,y,TOL).
% %
% %	[p,w] = nnls(X,y) also returns dual vector w where
% %	w(i) < 0 where x(i) = 0 and w(i) = 0 where x(i) > 0.
% %
% %	See also lsqnonneg and nnlsb
% 
% %	L. Shure 5-8-87
% %	Revised, 12-15-88,8-31-89 LS.
% %	(Partly) Copyright (c) 1984-94 by The MathWorks, Inc.
% 
% %	Modified by R. Bro 5-7-96 according to
% %       Bro R., de Jong S., Journal of Chemometrics, 1997, 11, 393-401
% % 	Corresponds to the FNNLSa algorithm in the paper
% %
% %
% %	Rasmus bro
% %	Chemometrics Group, Food Technology
% %	Dept. Dairy and Food Science
% %	Royal Vet. & Agricultural
% %	DK-1958 Frederiksberg C
% %	Denmark
% %	rb@kvl.dk
% %	http://newton.foodsci.kvl.dk/rasmus.html
% %
% % Modified by Emiel van Loon, july 2007 to match the input of nnls exactly!
% %
% 
% %  Reference:
% %  Lawson and Hanson, "Solving Least Squares Problems", Prentice-Hall, 1974.
% 
% % manage input -- the only modification by Emiel van Loon
% XtX = X'*X;
% Xty = X'*y(:);
% 
% % initialize variables
% if nargin < 3
%     tol = 10*eps*norm(XtX,1)*max(size(XtX));
% end
% [m,n] = size(XtX);
% P = zeros(1,n);
% Z = 1:n;
% x = P';
% ZZ=Z;
% w = Xty-XtX*x;
% 
% % set up iteration criterion
% iter = 0;
% itmax = 30*n;
% 
% % outer loop to put variables into set to hold positive coefficients
% while any(Z) && any(w(ZZ) > tol)
%     [wt,t] = max(w(ZZ));
%     t = ZZ(t);
%     P(1,t) = t;
%     Z(t) = 0;
%     PP = find(P);
%     ZZ = find(Z);
%     nzz = size(ZZ);
%     z(PP')=(Xty(PP)'/XtX(PP,PP)');
%     z(ZZ) = zeros(nzz(2),nzz(1))';
%     z=z(:);
%     % inner loop to remove elements from the positive set which no longer belong
% 
%     while any((z(PP) <= tol)) && iter < itmax
% 
%         iter = iter + 1;
%         QQ = find((z <= tol) & P');
%         alpha = min(x(QQ)./(x(QQ) - z(QQ)));
%         x = x + alpha*(z - x);
%         ij = find(abs(x) < tol & P' ~= 0);
%         Z(ij)=ij';
%         P(ij)=zeros(1,max(size(ij)));
%         PP = find(P);
%         ZZ = find(Z);
%         nzz = size(ZZ);
%         z(PP)=(Xty(PP)'/XtX(PP,PP)');
%         z(ZZ) = zeros(nzz(2),nzz(1));
%         z=z(:);
%     end
%     x = z;
%     w = Xty-XtX*x;
% end
% 
